@@ -8,8 +8,11 @@ use Mojo::mysql::Migrations;
 use Mojo::mysql::PubSub;
 use Mojo::URL;
 use Scalar::Util 'weaken';
+use SQL::Abstract;
 
+has abstract => sub { SQL::Abstract->new };
 has auto_migrate    => 0;
+has database_class  => 'Mojo::mysql::Database';
 has dsn             => 'dbi:mysql:dbname=test';
 has max_connections => 5;
 has migrations      => sub {
@@ -28,7 +31,7 @@ has pubsub => sub {
 };
 
 
-our $VERSION = '0.14';
+our $VERSION = '1.00';
 
 sub db {
   my $self = shift;
@@ -37,7 +40,7 @@ sub db {
   delete @$self{qw(pid queue)} unless ($self->{pid} //= $$) eq $$;
 
   my ($dbh, $handle) = @{$self->_dequeue};
-  return Mojo::mysql::Database->new(dbh => $dbh, handle => $handle, mysql => $self);
+  return $self->database_class->new(dbh => $dbh, handle => $handle, mysql => $self);
 }
 
 sub from_string {
@@ -86,9 +89,7 @@ sub _dequeue {
   # Maintain Commits with Mojo::mysql::Transaction
   $dbh->{AutoCommit} = 1;
 
-  ++$self->{migrated} and $self->migrations->migrate
-    if !$self->{migrated} and $self->auto_migrate;
-
+  $self->migrations->migrate if $self->auto_migrate and !$self->{migrated}++;
   $self->emit(connection => $dbh);
   [$dbh];
 }
@@ -135,6 +136,12 @@ Mojo::mysql - Mojolicious and Async MySQL
   say $db->query('insert into names (name) values (?)', 'Daniel')
     ->last_insert_id;
 
+  # Use SQL::Abstract to generate queries for you
+  $db->insert('names', {name => 'Isabel'});
+  say $db->select('names', undef, {name => 'Isabel'})->hash->{id};
+  $db->update('names', {name => 'Bel'}, {name => 'Isabel'});
+  $db->delete('names', {name => 'Bel'});
+
   # Select one row at a time
   my $results = $db->query('select * from names');
   while (my $next = $results->hash) {
@@ -239,6 +246,16 @@ Emitted when a new database connection has been established.
 
 L<Mojo::mysql> implements the following attributes.
 
+=head2 abstract
+
+  $abstract = $mysql->abstract;
+  $mysql    = $mysql->abstract(SQL::Abstract->new);
+
+L<SQL::Abstract> object used to generate CRUD queries for L<Mojo::mysql::Database>.
+
+  # Generate statements and bind values
+  my ($stmt, @bind) = $mysql->abstract->select('names');
+
 =head2 auto_migrate
 
   my $bool = $mysql->auto_migrate;
@@ -249,6 +266,14 @@ soon as the first database connection has been established.
 
 Defaults to false.
 
+=head2 database_class
+
+  $class = $mysql->database_class;
+  $mysql = $mysql->database_class("MyApp::Database");
+
+Class to be used by L</"db">, defaults to L<Mojo::mysql::Database>. Note that this
+class needs to have already been loaded before L</"db"> is called.
+
 =head2 dsn
 
   my $dsn = $mysql->dsn;
@@ -374,6 +399,13 @@ Parse configuration from connection string.
 Construct a new L<Mojo::mysql> object and parse connection string with
 L</"from_string"> if necessary.
 
+=head1 DEBUGGING
+
+You can set the C<DBI_TRACE> environment variable to get some advanced
+diagnostics information printed to C<STDERR> by L<DBI>.
+
+  DBI_TRACE=1
+
 =head1 REFERENCE
 
 This is the class hierarchy of the L<Mojo::mysql> distribution.
