@@ -1,13 +1,24 @@
 package Mojo::mysql::Database;
 use Mojo::Base 'Mojo::EventEmitter';
 
+use Carp;
 use DBD::mysql;
 use Mojo::IOLoop;
 use Mojo::mysql::Results;
 use Mojo::mysql::Transaction;
+use Mojo::Util 'monkey_patch';
 use Scalar::Util 'weaken';
 
 has [qw(dbh mysql)];
+has results_class => 'Mojo::mysql::Results';
+
+for my $name (qw(delete insert select update)) {
+  monkey_patch __PACKAGE__, $name, sub {
+    my $self = shift;
+    my @cb = ref $_[-1] eq 'CODE' ? pop : ();
+    return $self->query($self->mysql->abstract->$name(@_), @cb);
+  };
+}
 
 sub DESTROY {
   my $self = shift;
@@ -44,14 +55,15 @@ sub query {
   # Blocking
   unless ($cb) {
     my $sth = $self->dbh->prepare($query);
-    my $rv  = $sth->execute(@_);
-    my $res = Mojo::mysql::Results->new(sth => $sth);
+    local $sth->{HandleError} = sub { $_[0] = Carp::shortmess($_[0]); 0 };
+    my $rv = $sth->execute(@_);
+    my $res = $self->results_class->new(sth => $sth);
     $res->{affected_rows} = defined $rv && $rv >= 0 ? 0 + $rv : undef;
     return $res;
   }
 
   # Non-blocking
-  push @{$self->{waiting}}, {args => [@_], cb => $cb, query => $query};
+  push @{$self->{waiting}}, {args => [@_], err => Carp::shortmess('__MSG__'), cb => $cb, query => $query};
   $self->$_ for qw(_next _watch);
 }
 
@@ -95,12 +107,14 @@ sub _watch {
 
       return unless my $waiting = $self->{waiting};
       return unless @$waiting and $waiting->[0]{sth} and $waiting->[0]{sth}->mysql_async_ready;
-      my ($sth, $cb) = @{shift @$waiting}{qw(sth cb)};
+      my ($cb, $err, $sth) = @{shift @$waiting}{qw(cb err sth)};
 
       # Do not raise exceptions inside the event loop
       my $rv = do { local $sth->{RaiseError} = 0; $sth->mysql_async_result; };
-      my $err = defined $rv ? undef : $dbh->errstr;
-      my $res = Mojo::mysql::Results->new(sth => $sth);
+      my $res = $self->results_class->new(sth => $sth);
+
+      $err = undef if defined $rv;
+      $err =~ s!\b__MSG__\b!{$dbh->errstr}!e if defined $err;
       $res->{affected_rows} = defined $rv && $rv >= 0 ? 0 + $rv : undef;
 
       $self->$cb($err, $res);
@@ -146,6 +160,14 @@ Database handle used for all queries.
 
 L<Mojo::mysql> object this database belongs to.
 
+=head2 results_class
+
+  $class = $db->results_class;
+  $db    = $db->results_class("MyApp::Results");
+
+Class to be used by L</"query">, defaults to L<Mojo::mysql::Results>. Note that
+this class needs to have already been loaded before L</"query"> is called.
+
 =head1 METHODS
 
 L<Mojo::mysql::Database> inherits all methods from L<Mojo::EventEmitter> and
@@ -174,12 +196,40 @@ L<Mojo::mysql::Transaction/"commit"> bas been called before it is destroyed.
   };
   say $@ if $@;
 
+=head2 delete
+
+  my $results = $db->delete($table, \%where);
+
+Generate a C<DELETE> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->delete(some_table => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 disconnect
 
   $db->disconnect;
 
 Disconnect database handle and prevent it from getting cached again.
 
+=head2 insert
+
+  my $results = $db->insert($table, \@values || \%fieldvals, \%options);
+
+Generate an C<INSERT> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->insert(some_table => {foo => 'bar'} => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 pid
 
   my $pid = $db->pid;
@@ -218,6 +268,34 @@ Quote a string literal for use as a literal value in an SQL statement.
  
 Quote an identifier (table name etc.) for use in an SQL statement.
 
+=head2 select
+
+  my $results = $db->select($source, $fields, $where, $order);
+
+Generate a C<SELECT> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->select(some_table => ['foo'] => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
+=head2 update
+
+  my $results = $db->update($table, \%fieldvals, \%where);
+
+Generate an C<UPDATE> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->update(some_table => {foo => 'baz'} => {foo => 'bar'} => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head1 SEE ALSO
 
 L<Mojo::mysql>.
